@@ -1,6 +1,6 @@
 /* deflate.c -- compress data using the deflation algorithm
 
-   Copyright (C) 1999, 2006, 2009-2022 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2006, 2009-2025 Free Software Foundation, Inc.
    Copyright (C) 1992-1993 Jean-loup Gailly
 
    This program is free software; you can redistribute it and/or modify
@@ -14,8 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /*
  *  PURPOSE
@@ -68,7 +67,7 @@
  *      void lm_init (int pack_level, ush *flags)
  *          Initialize the "longest match" routines for a new file
  *
- *      off_t deflate (void)
+ *      off_t gzip_deflate ()
  *          Processes a new input file and return its compressed length. Sets
  *          the compressed length, crc, deflate flags and internal file
  *          attributes.
@@ -77,10 +76,10 @@
 #include <config.h>
 #include <stdio.h>
 
+#include "deflate.h"
 #include "tailor.h"
 #include "gzip.h"
 #include "lzw.h" /* just for consistency checking */
-#include "verify.h"
 
 /* ===========================================================================
  * Configuration parameters
@@ -131,7 +130,7 @@
 #ifndef RSYNC_WIN
 #  define RSYNC_WIN 4096
 #endif
-verify(RSYNC_WIN < MAX_DIST);
+static_assert (RSYNC_WIN < MAX_DIST);
 
 #define RSYNC_SUM_MATCH(sum) ((sum) % RSYNC_WIN == 0)
 /* Whether window sum matches magic value */
@@ -176,7 +175,7 @@ long block_start;
  * negative when the window is moved backwards.
  */
 
-local unsigned ins_h;  /* hash index of string to be inserted */
+static unsigned ins_h;  /* hash index of string to be inserted */
 
 #define H_SHIFT  ((HASH_BITS+MIN_MATCH-1)/MIN_MATCH)
 /* Number of bits by which ins_h and del_h must be shifted at each
@@ -185,22 +184,22 @@ local unsigned ins_h;  /* hash index of string to be inserted */
  *   H_SHIFT * MIN_MATCH >= HASH_BITS
  */
 
-       unsigned int near prev_length;
+unsigned int near prev_length;
 /* Length of the best match at previous step. Matches not greater than this
  * are discarded. This is used in the lazy match evaluation.
  */
 
-      unsigned near strstart;      /* start of string to insert */
-      unsigned near match_start;   /* start of matching string */
-local int           eofile;        /* flag set at end of input file */
-local unsigned      lookahead;     /* number of valid bytes ahead in window */
+unsigned near strstart;      /* start of string to insert */
+unsigned near match_start;   /* start of matching string */
+static int eofile;		   /* flag set at end of input file */
+static unsigned lookahead;	   /* number of valid bytes ahead in window */
 
-       unsigned max_chain_length;
+unsigned max_chain_length;
 /* To speed up deflation, hash chains are never searched beyond this length.
  * A higher limit improves compression ratio but degrades the speed.
  */
 
-local unsigned int max_lazy_match;
+static unsigned int max_lazy_match;
 /* Attempt to find a better match only when the current match is strictly
  * smaller than this value. This mechanism is used only for compression
  * levels >= 4.
@@ -214,8 +213,8 @@ local unsigned int max_lazy_match;
 unsigned good_match;
 /* Use a faster search when the previous match is longer than this */
 
-local ulg rsync_sum;  /* rolling sum of rsync window */
-local ulg rsync_chunk_end; /* next rsync sequence point */
+static ulg rsync_sum;  /* rolling sum of rsync window */
+static ulg rsync_chunk_end; /* next rsync sequence point */
 
 /* Values for max_lazy_match, good_match and max_chain_length, depending on
  * the desired pack level (0..9). The values given below have been tuned to
@@ -243,7 +242,7 @@ typedef struct config {
   static_unless_ASMV int nice_match;
 #endif
 
-local config configuration_table[10] = {
+static config configuration_table[10] = {
 /*      good lazy nice chain */
 /* 0 */ {0,    0,  0,    0},  /* store only */
 /* 1 */ {4,    4,  8,    4},  /* maximum speed, no lazy matches */
@@ -265,8 +264,8 @@ local config configuration_table[10] = {
 /* ===========================================================================
  *  Prototypes for local functions.
  */
-local void fill_window   (void);
-local off_t deflate_fast (void);
+static void fill_window (void);
+static off_t deflate_fast (void);
 
 #ifdef ASMV
       int  longest_match (IPos cur_match);
@@ -274,7 +273,7 @@ local off_t deflate_fast (void);
 #endif
 
 #ifdef DEBUG
-local  void check_match (IPos start, IPos match, int length);
+static void check_match (IPos start, IPos match, int length);
 #endif
 
 /* ===========================================================================
@@ -500,20 +499,19 @@ longest_match(IPos cur_match)
 /* ===========================================================================
  * Check that the match at match_start is indeed a match.
  */
-local void check_match(start, match, length)
-    IPos start, match;
-    int length;
+static void
+check_match (IPos start, IPos match, int length)
 {
     /* check that the match is indeed a match */
     if (memcmp((char*)window + match,
                 (char*)window + start, length) != 0) {
         fprintf(stderr,
-            " start %d, match %d, length %d\n",
+            " start %u, match %u, length %d\n",
             start, match, length);
         gzip_error ("invalid match");
     }
     if (verbose > 1) {
-        fprintf(stderr,"\\[%d,%d]", start-match, length);
+        fprintf (stderr, "\\[%u,%d]", start - match, length);
         do { putc(window[start++], stderr); } while (--length != 0);
     }
 }
@@ -529,7 +527,8 @@ local void check_match(start, match, length)
  *    file reads are performed for at least two bytes (required for the
  *    translate_eol option).
  */
-local void fill_window()
+static void
+fill_window ()
 {
     register unsigned n, m;
     unsigned more = (unsigned)(window_size - (ulg)lookahead - (ulg)strstart);
@@ -585,7 +584,8 @@ local void fill_window()
 
 /* With an initial offset of START, advance rsync's rolling checksum
    by NUM bytes.  */
-local void rsync_roll(unsigned int start, unsigned int num)
+static void
+rsync_roll (unsigned int start, unsigned int num)
 {
     unsigned i;
 
@@ -631,7 +631,8 @@ local void rsync_roll(unsigned int start, unsigned int num)
  * new strings in the dictionary only for unmatched strings or for short
  * matches. It is used only for the fast compression options.
  */
-local off_t deflate_fast()
+static off_t
+deflate_fast ()
 {
     IPos hash_head; /* head of the hash chain */
     int flush = 0;  /* set if current block must be flushed, 2=>and padded  */
@@ -720,7 +721,7 @@ local off_t deflate_fast()
  * no better match at the next window position.
  */
 off_t
-deflate (int pack_level)
+gzip_deflate (int pack_level)
 {
     IPos hash_head;          /* head of hash chain */
     IPos prev_match;         /* previous match */
