@@ -1,6 +1,6 @@
 /* gzip (GNU zip) -- compress files with zip algorithm and 'compress' interface
 
-   Copyright (C) 1999, 2001-2002, 2006-2007, 2009-2022 Free Software
+   Copyright (C) 1999, 2001-2002, 2006-2007, 2009-2025 Free Software
    Foundation, Inc.
    Copyright (C) 1992-1993 Jean-loup Gailly
 
@@ -15,8 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /*
  * The unzip code was written and put in the public domain by Mark Adler.
@@ -29,7 +28,7 @@
  */
 
 static char const *const license_msg[] = {
-"Copyright (C) 2018 Free Software Foundation, Inc.",
+"Copyright (C) 2025 Free Software Foundation, Inc.",
 "Copyright (C) 1993 Jean-loup Gailly.",
 "This is free software.  You may redistribute copies of it under the terms of",
 "the GNU General Public License <https://www.gnu.org/licenses/gpl.html>.",
@@ -58,10 +57,9 @@ static char const *const license_msg[] = {
 #include <ctype.h>
 #include <sys/types.h>
 #include <signal.h>
-#include <stdalign.h>
-#include <stdbool.h>
 #include <stddef.h>
 #include <sys/stat.h>
+#include <getopt.h>
 #include <errno.h>
 
 #include "tailor.h"
@@ -72,9 +70,8 @@ static char const *const license_msg[] = {
 #include "timespec.h"
 
 #include "dirname.h"
-#include "dosname.h"
 #include "fcntl--.h"
-#include "getopt.h"
+#include "filename.h"
 #include "ignore-value.h"
 #include "stat-time.h"
 #include "version.h"
@@ -84,9 +81,9 @@ static char const *const license_msg[] = {
                 /* configuration */
 
 #include <limits.h>
+#include <inttypes.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <errno.h>
 
 #ifndef NO_DIR
 # define NO_DIR 0
@@ -108,10 +105,6 @@ static char const *const license_msg[] = {
 #  define SEEK_END 2
 #endif
 
-#ifndef CHAR_BIT
-#  define CHAR_BIT 8
-#endif
-
 #ifdef off_t
   off_t lseek (int fd, off_t offset, int whence);
 #endif
@@ -134,7 +127,7 @@ static char const *const license_msg[] = {
    windows; the alignment requirement is 4096.  On other platforms
    alignment doesn't hurt, and alignment up to 4096 is portable so
    let's do that.  */
-#ifdef __alignas_is_defined
+#if defined HAVE_C_ALIGNASOF || defined alignas
 # define BUFFER_ALIGNED alignas (4096)
 #else
 # define BUFFER_ALIGNED /**/
@@ -204,11 +197,6 @@ struct timespec time_stamp;
 /* The set of signals that are caught.  */
 static sigset_t caught_signals;
 
-/* If nonzero then exit with status WARNING, rather than with the usual
-   signal status, on receipt of a signal with this value.  This
-   suppresses a "Broken Pipe" message with some shells.  */
-static int volatile exiting_signal;
-
 /* If nonnegative, close this file descriptor and unlink remove_ofname
    on error.  */
 static int volatile remove_ofname_fd = -1;
@@ -261,10 +249,6 @@ enum
   PRESUME_INPUT_TTY_OPTION = CHAR_MAX + 1,
   RSYNCABLE_OPTION,
   SYNCHRONOUS_OPTION,
-
-  /* A value greater than all valid long options, used as a flag to
-     distinguish options derived from the GZIP environment variable.  */
-  ENV_OPTION
 };
 
 static char const shortopts[] = "ab:cdfhH?klLmMnNqrS:tvVZ123456789";
@@ -289,7 +273,7 @@ static const struct option longopts[] =
     {"-presume-input-tty", no_argument, NULL, PRESUME_INPUT_TTY_OPTION},
     {"quiet",      0, 0, 'q'}, /* quiet mode */
     {"silent",     0, 0, 'q'}, /* quiet mode */
-    {"synchronous",0, 0, SYNCHRONOUS_OPTION},
+    {"synchronous",0, 0, SYNCHRONOUS_OPTION}, /* output data synchronously */
     {"recursive",  0, 0, 'r'}, /* recurse through directories */
     {"suffix",     1, 0, 'S'}, /* use given suffix instead of .gz */
     {"test",       0, 0, 't'}, /* test compressed file integrity */
@@ -305,33 +289,33 @@ static const struct option longopts[] =
 
 /* local functions */
 
-local noreturn void try_help (void);
-local void help         (void);
-local void license      (void);
-local void version      (void);
-local int input_eof	(void);
-local void treat_stdin  (void);
-local void treat_file   (char *iname);
-local int create_outfile (void);
-local char *get_suffix  (char *name);
-local int  open_input_file (char *iname, struct stat *sbuf);
-local void discard_input_bytes (size_t nbytes, unsigned int flags);
-local int  make_ofname  (void);
-local void shorten_name  (char *name);
-local int  get_method   (int in);
-local void do_list      (int method);
-local int  check_ofname (void);
-local void copy_stat    (struct stat *ifstat);
-local void install_signal_handlers (void);
+_Noreturn static void try_help (void);
+static void help (void);
+static void license (void);
+static void version (void);
+static int input_eof (void);
+static void treat_stdin (void);
+static void treat_file (char *iname);
+static int create_outfile (void);
+static char *get_suffix (char *name);
+static int  open_input_file (char *iname, struct stat *sbuf);
+static void discard_input_bytes (size_t nbytes, unsigned int flags);
+static int  make_ofname (void);
+static void shorten_name (char *name);
+static int  get_method (int in);
+static void do_list (int method);
+static int  check_ofname (void);
+static void copy_stat (struct stat *ifstat);
+static void install_signal_handlers (void);
 static void remove_output_file (bool);
 static void abort_gzip_signal (int);
-local noreturn void do_exit (int exitcode);
+_Noreturn static void do_exit (int exitcode);
 static void finish_out (void);
       int main          (int argc, char **argv);
 static int (*work) (int infile, int outfile) = zip; /* function to call */
 
 #if ! NO_DIR
-local void treat_dir    (int fd, char *dir);
+static void treat_dir (int fd, char *dir);
 #endif
 
 #define strequ(s1, s2) (strcmp((s1),(s2)) == 0)
@@ -345,7 +329,8 @@ try_help ()
 }
 
 /* ======================================================================== */
-local void help()
+static void
+help ()
 {
     static char const* const help_msg[] = {
  "Compress or uncompress FILEs (by default, compress FILES in-place).",
@@ -394,7 +379,8 @@ local void help()
 }
 
 /* ======================================================================== */
-local void license()
+static void
+license ()
 {
     char const *const *p = license_msg;
 
@@ -403,19 +389,18 @@ local void license()
 }
 
 /* ======================================================================== */
-local void version()
+static void
+version ()
 {
     license ();
     printf ("\n");
     printf ("Written by Jean-loup Gailly.\n");
 }
 
-local void progerror (char const *string)
+static void
+progerror (char const *string)
 {
-    int e = errno;
-    fprintf (stderr, "%s: ", program_name);
-    errno = e;
-    perror(string);
+    fprintf (stderr, "%s: %s: %s\n", program_name, string, strerror (errno));
     exit_code = ERROR;
 }
 
@@ -441,6 +426,7 @@ int main (int argc, char **argv)
     argv_copy = argv;
     env = add_envopt (&env_argc, &argv_copy, OPTIONS_VAR);
     env_argv = env ? argv_copy : NULL;
+    opterr = !env;
 
 #ifndef GNU_STANDARD
 # define GNU_STANDARD 1
@@ -466,48 +452,35 @@ int main (int argc, char **argv)
 
     while (true) {
         int optc;
-        int longind = -1;
 
         if (env_argv)
           {
-            if (env_argv[optind] && strequ (env_argv[optind], "--"))
-              optc = ENV_OPTION + '-';
-            else
+            /* Get the next option taken from the GZIP environment variable
+               that affects only performance (including compression level).
+               Silently ignore other options, non-options, invalid options,
+               and option syntax errors, so that bad GZIP values do
+               not unduly interfere with normal gzip operation.  */
+            do
               {
                 optc = getopt_long (env_argc, env_argv, shortopts, longopts,
-                                    &longind);
-                if (0 <= optc)
-                  optc += ENV_OPTION;
-                else
+                                    NULL);
+                if (optc < 0)
                   {
-                    if (optind != env_argc)
-                      {
-                        fprintf (stderr,
-                                 ("%s: %s: non-option in "OPTIONS_VAR
-                                  " environment variable\n"),
-                                 program_name, env_argv[optind]);
-                        try_help ();
-                      }
-
-                    /* Wait until here before warning, so that GZIP='-q'
-                       doesn't warn.  */
-                    if (env_argc != 1 && !quiet)
-                      fprintf (stderr,
-                               ("%s: warning: "OPTIONS_VAR" environment variable"
-                                " is deprecated; use an alias or script\n"),
-                               program_name);
-
                     /* Start processing ARGC and ARGV instead.  */
                     free (env_argv);
                     env_argv = NULL;
+                    opterr = 1;
                     optind = 1;
-                    longind = -1;
+                    break;
                   }
               }
+            while (! (('1' <= optc && optc <= '9')
+                      || optc == RSYNCABLE_OPTION
+                      || optc == SYNCHRONOUS_OPTION));
           }
 
         if (!env_argv)
-          optc = getopt_long (argc, argv, shortopts, longopts, &longind);
+          optc = getopt_long (argc, argv, shortopts, longopts, NULL);
         if (optc < 0)
           break;
 
@@ -543,15 +516,12 @@ int main (int argc, char **argv)
         case 'M': /* undocumented, may change later */
             no_time = 0; break;
         case 'n':
-        case 'n' + ENV_OPTION:
             no_name = no_time = 1; break;
         case 'N':
-        case 'N' + ENV_OPTION:
             no_name = no_time = 0; break;
         case PRESUME_INPUT_TTY_OPTION:
             presume_input_tty = true; break;
         case 'q':
-        case 'q' + ENV_OPTION:
             quiet = 1; verbose = 0; break;
         case 'r':
 #if NO_DIR
@@ -564,14 +534,18 @@ int main (int argc, char **argv)
             break;
 
         case RSYNCABLE_OPTION:
-        case RSYNCABLE_OPTION + ENV_OPTION:
             rsync = 1;
             break;
         case 'S':
 #ifdef NO_MULTIPLE_DOTS
             if (*optarg == '.') optarg++;
 #endif
-            z_len = strlen(optarg);
+            for (z_len = 0; optarg[z_len]; z_len++)
+              if (optarg[z_len] == '/')
+                {
+                  fprintf (stderr, "%s: suffix contains '/'\n", program_name);
+                  do_exit (ERROR);
+                }
             z_suffix = optarg;
             break;
         case SYNCHRONOUS_OPTION:
@@ -581,7 +555,6 @@ int main (int argc, char **argv)
             test = decompress = to_stdout = 1;
             break;
         case 'v':
-        case 'v' + ENV_OPTION:
             verbose++; quiet = 0; break;
         case 'V':
             version (); finish_out (); break;
@@ -590,28 +563,12 @@ int main (int argc, char **argv)
                     program_name);
             try_help ();
             break;
-        case '1' + ENV_OPTION:  case '2' + ENV_OPTION:  case '3' + ENV_OPTION:
-        case '4' + ENV_OPTION:  case '5' + ENV_OPTION:  case '6' + ENV_OPTION:
-        case '7' + ENV_OPTION:  case '8' + ENV_OPTION:  case '9' + ENV_OPTION:
-            optc -= ENV_OPTION;
-            FALLTHROUGH;
         case '1':  case '2':  case '3':  case '4':
         case '5':  case '6':  case '7':  case '8':  case '9':
             level = optc - '0';
             break;
 
         default:
-            if (ENV_OPTION <= optc && optc != ENV_OPTION + '?')
-              {
-                /* Output a diagnostic, since getopt_long didn't.  */
-                fprintf (stderr, "%s: ", program_name);
-                if (longind < 0)
-                  fprintf (stderr, "-%c: ", optc - ENV_OPTION);
-                else
-                  fprintf (stderr, "--%s: ", longopts[longind].name);
-                fprintf (stderr, ("option not valid in "OPTIONS_VAR
-                                  " environment variable\n"));
-              }
             try_help ();
         }
     } /* loop on all arguments */
@@ -648,11 +605,6 @@ int main (int argc, char **argv)
     ALLOC(ush, tab_prefix1, 1L<<(BITS-1));
 #endif
 
-#if SIGPIPE
-    exiting_signal = quiet ? SIGPIPE : 0;
-#endif
-    install_signal_handlers ();
-
     /* And get to work */
     if (file_count != 0) {
         if (to_stdout && !test && (!decompress || !ascii)) {
@@ -687,7 +639,7 @@ int main (int argc, char **argv)
 }
 
 /* Return nonzero when at end of file on input.  */
-local int
+static int
 input_eof ()
 {
   if (!decompress || last_member)
@@ -727,7 +679,8 @@ get_input_size_and_time (void)
 /* ========================================================================
  * Compress or decompress stdin
  */
-local void treat_stdin()
+static void
+treat_stdin ()
 {
     if (!force && !list
         && (presume_input_tty
@@ -798,7 +751,6 @@ local void treat_stdin()
 
         method = get_method(ifd);
         if (method < 0) return; /* error message already emitted */
-        bytes_out = 0;            /* required for length check */
     }
 
     if (list)
@@ -868,8 +820,8 @@ atdir_set (char const *dir, ptrdiff_t dirlen)
 /* ========================================================================
  * Compress or decompress the given file
  */
-local void treat_file(iname)
-    char *iname;
+static void
+treat_file (char *iname)
 {
     /* Accept "-" as synonym for stdin */
     if (strequ(iname, "-")) {
@@ -1005,7 +957,6 @@ local void treat_file(iname)
 
         method = get_method(ifd);
         if (method < 0) break;    /* error message already emitted */
-        bytes_out = 0;            /* required for length check */
     }
 
     if (close (ifd) != 0)
@@ -1042,14 +993,8 @@ local void treat_file(iname)
             sigprocmask (SIG_SETMASK, &oldset, NULL);
 
             if (unlink_errno)
-              {
-                WARN ((stderr, "%s: ", program_name));
-                if (!quiet)
-                  {
-                    errno = unlink_errno;
-                    perror (ifname);
-                  }
-              }
+              WARN ((stderr, "%s: %s: %s\n", program_name, ifname,
+                     strerror (unlink_errno)));
           }
       }
 
@@ -1091,8 +1036,10 @@ volatile_strcpy (char volatile *dst, char const volatile *src)
  *   ofname has already been updated if there was an original name.
  * OUT assertions: ifd and ofd are closed in case of error.
  */
-local int create_outfile()
+static int
+create_outfile ()
 {
+  static bool signal_handlers_installed;
   int name_shortened = 0;
   int flags = (O_WRONLY | O_CREAT | O_EXCL
                | (ascii && decompress ? 0 : O_BINARY));
@@ -1110,6 +1057,12 @@ local int create_outfile()
         }
     }
 
+  if (!signal_handlers_installed)
+    {
+      signal_handlers_installed = true;
+      install_signal_handlers ();
+    }
+
   for (;;)
     {
       int open_errno;
@@ -1143,7 +1096,7 @@ local int create_outfile()
           break;
 
         default:
-          progerror (ofname);
+          write_error ();
           close (ifd);
           return ERROR;
         }
@@ -1169,8 +1122,8 @@ local int create_outfile()
  * .??z suffix as indicating a compressed file; some people use .xyz
  * to denote volume data.
  */
-local char *get_suffix(name)
-    char *name;
+static char *
+get_suffix (char *name)
 {
     int nlen, slen;
     char suffix[MAX_SUFFIX+3]; /* last chars of name, forced to lower case */
@@ -1285,9 +1238,7 @@ open_and_stat (char *name, int flags, struct stat *st)
  * Return an open file descriptor or -1.
  */
 static int
-open_input_file (iname, sbuf)
-    char *iname;
-    struct stat *sbuf;
+open_input_file (char *iname, struct stat *sbuf)
 {
     int ilen;  /* strlen(ifname) */
     int z_suffix_errno = 0;
@@ -1384,7 +1335,8 @@ open_input_file (iname, sbuf)
  * Generate ofname given ifname. Return OK, or WARNING if file must be skipped.
  * Sets save_orig_name to true if the file name has been truncated.
  */
-local int make_ofname()
+static int
+make_ofname ()
 {
     char *suff;            /* ofname z suffix */
 
@@ -1466,9 +1418,7 @@ local int make_ofname()
    zero byte if NBYTES == (size_t) -1.  If FLAGS say that the header
    CRC should be computed, update the CRC accordingly.  */
 static void
-discard_input_bytes (nbytes, flags)
-    size_t nbytes;
-    unsigned int flags;
+discard_input_bytes (size_t nbytes, unsigned int flags)
 {
   while (nbytes != 0)
     {
@@ -1490,11 +1440,12 @@ discard_input_bytes (nbytes, flags)
  * Updates time_stamp if there is one and neither -m nor -n is used.
  * This function may be called repeatedly for an input file consisting
  * of several contiguous gzip'ed members.
+ * 'in' is the input file descriptor.
  * IN assertions: there is at least one remaining compressed member.
  *   If the member is a zip file, it must be the only one.
  */
-local int get_method(in)
-    int in;        /* input file descriptor */
+static int
+get_method (int in)
 {
     uch flags;     /* compression flags */
     uch magic[10]; /* magic header */
@@ -1755,12 +1706,9 @@ do_list (int method)
         if (verbose) {
             printf("                            ");
         }
-        if (verbose || !quiet) {
-            fprint_off(stdout, total_in, positive_off_t_width);
-            printf(" ");
-            fprint_off(stdout, total_out, positive_off_t_width);
-            printf(" ");
-        }
+        if (verbose || !quiet)
+          printf ("%*jd %*jd ", positive_off_t_width, (intmax_t) total_in,
+                  positive_off_t_width, (intmax_t) total_out);
         display_ratio(total_out-(total_in-header_bytes), total_out, stdout);
         /* header_bytes is not meaningful but used to ensure the same
          * ratio if there is a single file.
@@ -1787,10 +1735,8 @@ do_list (int method)
         else
           printf ("??? ?? ??:?? ");
       }
-    fprint_off(stdout, bytes_in, positive_off_t_width);
-    printf(" ");
-    fprint_off(stdout, bytes_out, positive_off_t_width);
-    printf(" ");
+    printf ("%*jd %*jd ", positive_off_t_width, (intmax_t) bytes_in,
+            positive_off_t_width, (intmax_t) bytes_out);
     if (bytes_in  == -1L) {
         total_in = -1L;
         bytes_in = bytes_out = header_bytes = 0;
@@ -1816,8 +1762,8 @@ do_list (int method)
  *
  * IN assertion: for compression, the suffix of the given name is z_suffix.
  */
-local void shorten_name(name)
-    char *name;
+static void
+shorten_name (char *name)
 {
     int len;                 /* length of name without z_suffix */
     char *trunc = NULL;      /* character to be truncated */
@@ -1874,7 +1820,8 @@ local void shorten_name(name)
  * The compressed file already exists, so ask for confirmation.
  * Return ERROR if the file must be skipped.
  */
-local int check_ofname()
+static int
+check_ofname ()
 {
     /* Ask permission to overwrite the existing file */
     if (!force) {
@@ -1921,8 +1868,8 @@ do_chown (int fd, char const *name, uid_t uid, gid_t gid)
  * Copy modes, times, ownership from input file to output file.
  * IN assertion: to_stdout is false.
  */
-local void copy_stat(ifstat)
-    struct stat *ifstat;
+static void
+copy_stat (struct stat *ifstat)
 {
     mode_t mode = ifstat->st_mode & S_IRWXUGO;
     int r;
@@ -1945,15 +1892,7 @@ local void copy_stat(ifstat)
         }
       }
     else
-      {
-        int e = errno;
-        WARN ((stderr, "%s: ", program_name));
-        if (!quiet)
-          {
-            errno = e;
-            perror (ofname);
-          }
-      }
+      WARN ((stderr, "%s: %s: %s\n", program_name, ofname, strerror (errno)));
 #endif
 
     /* Change the group first, then the permissions, then the owner.
@@ -1969,14 +1908,8 @@ local void copy_stat(ifstat)
 #else
     r = chmod (ofname, mode);
 #endif
-    if (r != 0) {
-        int e = errno;
-        WARN ((stderr, "%s: ", program_name));
-        if (!quiet) {
-            errno = e;
-            perror(ofname);
-        }
-    }
+    if (r != 0)
+      WARN ((stderr, "%s: %s: %s\n", program_name, ofname, strerror (errno)));
 
     do_chown (ofd, ofname, ifstat->st_uid, -1);
 }
@@ -1986,9 +1919,8 @@ local void copy_stat(ifstat)
 /* ========================================================================
  * Recurse through the given directory.
  */
-local void treat_dir (fd, dir)
-    int fd;
-    char *dir;
+static void
+treat_dir (int fd, char *dir)
 {
     DIR      *dirp;
     char     nbuf[MAX_PATH_LEN];
@@ -2066,8 +1998,8 @@ install_signal_handlers ()
 /* ========================================================================
  * Free all dynamically allocated variables and exit with the given code.
  */
-local void do_exit(exitcode)
-    int exitcode;
+static void
+do_exit (int exitcode)
 {
     static int in_exit = 0;
 
@@ -2089,7 +2021,7 @@ local void do_exit(exitcode)
 }
 
 static void
-finish_out (void)
+finish_out ()
 {
   if (fclose (stdout) != 0)
     write_error ();
@@ -2124,12 +2056,17 @@ remove_output_file (bool signals_already_blocked)
  * Error handler.
  */
 void
-abort_gzip (void)
+finish_up_gzip (int exitcode)
 {
-   remove_output_file (false);
-   do_exit(ERROR);
+  if (0 <= remove_ofname_fd)
+    remove_output_file (false);
+  do_exit (exitcode);
+}
+void
+abort_gzip ()
+{
+  finish_up_gzip (ERROR);
 }
-
 /* ========================================================================
  * Signal handler.
  */
@@ -2137,8 +2074,6 @@ static void
 abort_gzip_signal (int sig)
 {
    remove_output_file (true);
-   if (sig == exiting_signal)
-     _exit (WARNING);
    signal (sig, SIG_DFL);
    raise (sig);
 }
